==> nui_mini_program-1.0.5.tar.gz: lib/util/nui_miniprogram_action_util.dart <==
import 'dart:convert';
import 'dart:io';
import 'package:file_picker/file_picker.dart';
import 'package:flutter/cupertino.dart';
import 'package:image_picker/image_picker.dart';
import 'package:nui_core/code/nui_code_util.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_core/ui/component/nui_toast.dart';
import 'package:nui_core/ui/nui_navigator.dart';
import 'package:nui_miniprogram_core/data/nui_miniprogram_session.dart';
import 'package:nui_miniprogram_core/nui_miniprogram_enum.dart';
import 'package:nui_miniprogram_core/nui_miniprogram_message.dart';
import 'package:path/path.dart';
import '../widget/nui_miniprogram_widget.dart';

/**
 * Utility class for processing mini-app actions
 */
class NUIMiniProgramActionUtil{

  /// Processing the action
  Future<NUIMiniProgramMessage?> processAction(
      BuildContext context,
      String message,
      NUIMiniProgramMessage msg,
      {
        Function? initializeListener,
        required NUIMiniProgramSessionBuilder sessionBuilder, // For session exchange
        required NUIMiniProgramMessageExchangeBuilder messageExchangeBuilder, // For custom message exchange
      }) async{

    try {
      final internalMessage = NUIMiniProgramInternalMessage.empty().fromJson(jsonDecode(message));
      switch(internalMessage.type){
        case NUIMiniProgramEventType.SESSION: {
          // Processing the current session details and return to mini-app
          final session = await _session(sessionBuilder);
          return msg.response(data: session);
        }
        case NUIMiniProgramEventType.POP: {
          // Popping the current screen
          pop(context);
          return null;
        }
        case NUIMiniProgramEventType.TOAST: {
          // Toasting a message
          toast(message, context: context);
          return null;
        }
        case NUIMiniProgramEventType.FILE_SELECT: {
          // Performing file selection
          final selectedFile = await _selectFile();
          return msg.response(data: selectedFile);
        }
        case NUIMiniProgramEventType.HIDE_KEYBOARD: {
          // Hiding the keyboard
          FocusScope.of(context).unfocus();
          return null;
        }
        case NUIMiniProgramEventType.STATUS_BAR_LIGHT: {
          // Changing the status bar style to light
          NUIDisplayUtil.setStatusBarStyleAttr(NUIOverlayStyle(darkStatusBarIcon: false), internal: true);
          return null;
        }
        case NUIMiniProgramEventType.STATUS_BAR_DARK: {
          // Changing the status bar style to dark
          NUIDisplayUtil.setStatusBarStyleAttr(NUIOverlayStyle(darkStatusBarIcon: true), internal: true);
          return null;
        }
        case NUIMiniProgramEventType.INITIALIZED: {
          // Sending the initialized event
          if(initializeListener != null) initializeListener();
          return null;
        }
        case NUIMiniProgramEventType.CAMERA: {
          // Taking a photo
          final selectedFile = await _takePhoto(rear: match(internalMessage.data, "true"));
          return msg.response(data: selectedFile);
        }
        case NUIMiniProgramEventType.VIDEO: {
          // Capturing a video
          final selectedFile = await _captureVideo(rear: match(internalMessage.data, "true"));
          return msg.response(data: selectedFile);
        }
        case NUIMiniProgramEventType.CUSTOM: {
          // Custom message exchange
          final result = await _messageExchange(msg, messageExchangeBuilder);
          return msg.response(data: result);
        }
      }
    }catch(e, s){
      logNUI("NUIMiniProgramActionUtil", "Failed to parse NUIMiniProgramInternalMessage with error : $e, stacktrace: $s");
    }
    return null;
  }

  /// Selecting a file
  Future<NUIMiniProgramFile?> _selectFile() async{
    FilePickerResult? result = await FilePicker.platform.pickFiles(
      // allowedExtensions: ['jpg', 'pdf', 'doc', 'png'],
    );

    if(result != null) {
      final selected = result.files[0];
      print("NUIMiniProgramActionUtil: Selected file : $selected, filename: ${selected.name}, path : ${selected.path}, bytes: ${selected.bytes?.length}");
      if(selected.bytes != null) {
        return NUIMiniProgramFile(filename: selected.name, bytes: selected.bytes, extensions: selected.extension != null ? [selected.extension!] : []);
      }
      else{
        try{
          final file = File(selected.path!);
          final bytes = await file.readAsBytes();
          return NUIMiniProgramFile(filename: selected.name, bytes: bytes, extensions: selected.extension != null ? [selected.extension!] : []);
        }catch(e, s){
          print("NUIMiniProgramActionUtil: Processed bytes failed with error: $e, stacktrace: $s");
        }
        return null;
      }
    } else {
      return null;
    }
  }

  /// Requesting a session
  Future<NUIMiniProgramSession> _session(NUIMiniProgramSessionBuilder sessionBuilder) async{
    NUIMiniProgramSession result = await sessionBuilder();
    return result;
  }


  /// Requesting a session
  Future<dynamic> _messageExchange(NUIMiniProgramMessage inputMessage, NUIMiniProgramMessageExchangeBuilder messageBuilder) async{
    // Process the response message based on the input message
    dynamic result = await messageBuilder(inputMessage.data);
    return result;
  }

  /// Taking a photo
  Future<NUIMiniProgramFile?> _takePhoto({bool rear: true}) async{
    final pickedFile = await ImagePicker().getImage(source: ImageSource.camera, preferredCameraDevice: rear ? CameraDevice.rear : CameraDevice.front);

    if (pickedFile != null) {
      try {
        final file = File(pickedFile.path);
        final bytes = await file.readAsBytes();
        return NUIMiniProgramFile(filename: basename(file.path), bytes: bytes, extensions: []);
      }catch(e, s){
        print("NUIMiniProgramActionUtil: Processed take photo bytes failed with error: $e, stacktrace: $s");
      }
    }
    return null;
  }

  /// Capturing a video
  Future<NUIMiniProgramFile?> _captureVideo({bool rear: true}) async{
    final pickedFile = await ImagePicker().getVideo(source: ImageSource.camera, preferredCameraDevice: rear ? CameraDevice.rear : CameraDevice.front);

    if (pickedFile != null) {
      try {
        final file = File(pickedFile.path);
        final bytes = await file.readAsBytes();
        return NUIMiniProgramFile(filename: basename(file.path), bytes: bytes, extensions: []);
      }catch(e, s){
        print("NUIMiniProgramActionUtil: Processed capture video bytes failed with error: $e, stacktrace: $s");
      }
    }
    return null;
  }
}

==> nui_mini_program-1.0.8.tar.gz: lib/util/nui_miniprogram_action_util.dart <==
import 'dart:convert';
import 'dart:io';
import 'package:file_picker/file_picker.dart';
import 'package:flutter/cupertino.dart';
import 'package:image_picker/image_picker.dart';
import 'package:nui_core/code/nui_code_util.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_core/ui/component/nui_toast.dart';
import 'package:nui_core/ui/nui_navigator.dart';
import 'package:nui_miniprogram_core/data/nui_miniprogram_session.dart';
import 'package:nui_miniprogram_core/nui_miniprogram_enum.dart';
import 'package:nui_miniprogram_core/nui_miniprogram_message.dart';
import 'package:path/path.dart';
import '../widget/nui_miniprogram_widget.dart';

/**
 * Utility class for processing mini-app actions
 */
class NUIMiniProgramActionUtil{

  /// Processing the action
  Future<NUIMiniProgramMessage?> processAction(
      BuildContext context,
      String message,
      NUIMiniProgramMessage msg,
      {
        Function? initializeListener,
        required NUIMiniProgramSessionBuilder sessionBuilder, // For session exchange
        required NUIMiniProgramMessageExchangeBuilder messageExchangeBuilder, // For custom message exchange
      }) async{

    try {
      final internalMessage = NUIMiniProgramInternalMessage.empty().fromJson(jsonDecode(message));
      switch(internalMessage.type){
        case NUIMiniProgramEventType.SESSION: {
          // Processing the current session details and return to mini-app
          final session = await _session(sessionBuilder);
          return msg.response(data: session);
        }
        case NUIMiniProgramEventType.POP: {
          // Popping the current screen
          pop(context);
          return null;
        }
        case NUIMiniProgramEventType.TOAST: {
          // Toasting a message
          toast(message, context: context);
          return null;
        }
        case NUIMiniProgramEventType.FILE_SELECT: {
          // Performing file selection
          final selectedFile = await _selectFile();
          return msg.response(data: selectedFile);
        }
        case NUIMiniProgramEventType.HIDE_KEYBOARD: {
          // Hiding the keyboard
          FocusScope.of(context).unfocus();
          return null;
        }
        case NUIMiniProgramEventType.STATUS_BAR_LIGHT: {
          // Changing the status bar style to light
          NUIDisplayUtil.setStatusBarStyleAttr(NUIOverlayStyle(darkStatusBarIcon: false), internal: true);
          return null;
        }
        case NUIMiniProgramEventType.STATUS_BAR_DARK: {
          // Changing the status bar style to dark
          NUIDisplayUtil.setStatusBarStyleAttr(NUIOverlayStyle(darkStatusBarIcon: true), internal: true);
          return null;
        }
        case NUIMiniProgramEventType.INITIALIZED: {
          // Sending the initialized event
          if(initializeListener != null) initializeListener();
          return null;
        }
        case NUIMiniProgramEventType.CAMERA: {
          // Taking a photo
          final selectedFile = await _takePhoto(rear: match(internalMessage.data, "true"));
          return msg.response(data: selectedFile);
        }
        case NUIMiniProgramEventType.VIDEO: {
          // Capturing a video
          final selectedFile = await _captureVideo(rear: match(internalMessage.data, "true"));
          return msg.response(data: selectedFile);
        }
        case NUIMiniProgramEventType.CUSTOM: {
          // Custom message exchange
          final result = await _messageExchange(jsonDecode(internalMessage.data), messageExchangeBuilder);
          return msg.response(data: result);
        }
      }
    }catch(e, s){
      logNUI("NUIMiniProgramActionUtil", "Failed to parse NUIMiniProgramInternalMessage with error : $e, stacktrace: $s");
    }
    return null;
  }

  /// Selecting a file
  Future<NUIMiniProgramFile?> _selectFile() async{
    FilePickerResult? result = await FilePicker.platform.pickFiles(
      // allowedExtensions: ['jpg', 'pdf', 'doc', 'png'],
    );

    if(result != null) {
      final selected = result.files[0];
      print("NUIMiniProgramActionUtil: Selected file : $selected, filename: ${selected.name}, path : ${selected.path}, bytes: ${selected.bytes?.length}");
      if(selected.bytes != null) {
        return NUIMiniProgramFile(filename: selected.name, bytes: selected.bytes, extensions: selected.extension != null ? [selected.extension!] : []);
      }
      else{
        try{
          final file = File(selected.path!);
          final bytes = await file.readAsBytes();
          return NUIMiniProgramFile(filename: selected.name, bytes: bytes, extensions: selected.extension != null ? [selected.extension!] : []);
        }catch(e, s){
          print("NUIMiniProgramActionUtil: Processed bytes failed with error: $e, stacktrace: $s");
        }
        return null;
      }
    } else {
      return null;
    }
  }

  /// Requesting a session
  Future<NUIMiniProgramSession> _session(NUIMiniProgramSessionBuilder sessionBuilder) async{
    NUIMiniProgramSession result = await sessionBuilder();
    return result;
  }


  /// Requesting a session
  Future<dynamic> _messageExchange(dynamic data, NUIMiniProgramMessageExchangeBuilder messageBuilder) async{
    // Process the response message based on the input message
    dynamic result = await messageBuilder(data);
    return result;
  }

  /// Taking a photo
  Future<NUIMiniProgramFile?> _takePhoto({bool rear: true}) async{
    final pickedFile = await ImagePicker().pickImage(source: ImageSource.camera, preferredCameraDevice: rear ? CameraDevice.rear : CameraDevice.front);

    if (pickedFile != null) {
      try {
        final file = File(pickedFile.path);
        final bytes = await file.readAsBytes();
        return NUIMiniProgramFile(filename: basename(file.path), bytes: bytes, extensions: []);
      }catch(e, s){
        print("NUIMiniProgramActionUtil: Processed take photo bytes failed with error: $e, stacktrace: $s");
      }
    }
    return null;
  }

  /// Capturing a video
  Future<NUIMiniProgramFile?> _captureVideo({bool rear: true}) async{
    final pickedFile = await ImagePicker().pickVideo(source: ImageSource.camera, preferredCameraDevice: rear ? CameraDevice.rear : CameraDevice.front);

    if (pickedFile != null) {
      try {
        final file = File(pickedFile.path);
        final bytes = await file.readAsBytes();
        return NUIMiniProgramFile(filename: basename(file.path), bytes: bytes, extensions: []);
      }catch(e, s){
        print("NUIMiniProgramActionUtil: Processed capture video bytes failed with error: $e, stacktrace: $s");
      }
    }
    return null;
  }
}
